@@ -1,10 +1,10 @@
-use std::cell::RefCell;
 use std::net::Shutdown;
-use std::rc::Rc;
+use std::sync::Arc;
 
 use bstr::BString;
 use tokio::net::{TcpListener, TcpStream};
 use tokio::prelude::*;
+use tokio::sync::Mutex;
 use tokio::task;
 
 use mlua::{Function, Lua, Result, UserData, UserDataMethods};
@@ -13,16 +13,16 @@ use mlua::{Function, Lua, Result, UserData, UserDataMethods};
 struct LuaTcp;
 
 #[derive(Clone)]
-struct LuaTcpListener(Rc<RefCell<TcpListener>>);
+struct LuaTcpListener(Arc<Mutex<TcpListener>>);
 
 #[derive(Clone)]
-struct LuaTcpStream(Rc<RefCell<TcpStream>>);
+struct LuaTcpStream(Arc<Mutex<TcpStream>>);
 
 impl UserData for LuaTcp {
     fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
         methods.add_async_function("bind", |_, addr: String| async move {
             let listener = TcpListener::bind(addr).await?;
-            Ok(LuaTcpListener(Rc::new(RefCell::new(listener))))
+            Ok(LuaTcpListener(Arc::new(Mutex::new(listener))))
         });
     }
 }
@@ -30,8 +30,8 @@ impl UserData for LuaTcp {
 impl UserData for LuaTcpListener {
     fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
         methods.add_async_method("accept", |_, listener, ()| async move {
-            let (stream, _) = listener.0.borrow_mut().accept().await?;
-            Ok(LuaTcpStream(Rc::new(RefCell::new(stream))))
+            let (stream, _) = listener.0.lock().await.accept().await?;
+            Ok(LuaTcpStream(Arc::new(Mutex::new(stream))))
         });
     }
 }
@@ -39,32 +39,29 @@ impl UserData for LuaTcpListener {
 impl UserData for LuaTcpStream {
     fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
         methods.add_async_method("peer_addr", |_, stream, ()| async move {
-            Ok(stream.0.borrow().peer_addr()?.to_string())
+            Ok(stream.0.lock().await.peer_addr()?.to_string())
         });
 
         methods.add_async_method("read", |_, stream, size: usize| async move {
             let mut buf = vec![0; size];
-            let n = stream.0.borrow_mut().read(&mut buf).await?;
+            let n = stream.0.lock().await.read(&mut buf).await?;
             buf.truncate(n);
             Ok(BString::from(buf))
         });
 
         methods.add_async_method("write", |_, stream, data: BString| async move {
-            let n = stream.0.borrow_mut().write(&data).await?;
+            let n = stream.0.lock().await.write(&data).await?;
             Ok(n)
         });
 
-        methods.add_method("close", |_, stream, ()| {
-            stream.0.borrow().shutdown(Shutdown::Both)?;
+        methods.add_async_method("close", |_, stream, ()| async move {
+            stream.0.lock().await.shutdown(Shutdown::Both)?;
             Ok(())
         });
     }
 }
 
-#[tokio::main]
-async fn main() -> Result<()> {
-    let lua = Lua::new();
-
+async fn run_server(lua: &'static Lua) -> Result<()> {
     let spawn = lua.create_function(move |_, func: Function| {
         task::spawn_local(async move { func.call_async::<_, ()>(()).await.unwrap() });
         Ok(())
@@ -80,20 +77,33 @@ async fn main() -> Result<()> {
             local addr = ...
             local listener = tcp.bind(addr)
             print("listening on "..addr)
+
+            local accept_new = true
             while true do
                 local stream = listener:accept()
                 local peer_addr = stream:peer_addr()
                 print("connected from "..peer_addr)
+
+                if not accept_new then
+                    return
+                end
+
                 spawn(function()
                     while true do
                         local data = stream:read(100)
                         data = data:match("^%s*(.-)%s*$") -- trim
                         print("["..peer_addr.."] "..data)
-                        stream:write("got: "..data.."\n")
+                        if data == "bye" then
+                            stream:write("bye bye\n")
+                            stream:close()
+                            return
+                        end
                         if data == "exit" then
                             stream:close()
-                            break
+                            accept_new = false
+                            return
                         end
+                        stream:write("echo: "..data.."\n")
                     end
                 end)
             end
@@ -105,3 +115,15 @@ async fn main() -> Result<()> {
         .run_until(server.call_async::<_, ()>("0.0.0.0:1234"))
         .await
 }
+
+#[tokio::main]
+async fn main() {
+    let lua = Lua::new().into_static();
+
+    run_server(lua).await.unwrap();
+
+    // Consume the static reference and drop it.
+    // This is safe as long as we don't hold any other references to Lua
+    // or alive resources.
+    unsafe { Lua::from_static(lua) };
+}
