@@ -15,6 +15,7 @@ use {futures_core::future::LocalBoxFuture, futures_util::future};
 #[derive(Clone, Debug)]
 pub struct Table<'lua>(pub(crate) LuaRef<'lua>);
 
+#[allow(clippy::len_without_is_empty)]
 impl<'lua> Table<'lua> {
     /// Sets a key-value pair in the table.
     ///
@@ -136,28 +137,6 @@ impl<'lua> Table<'lua> {
         }
     }
 
-    /// Gets the function associated to `key` from the table and executes it,
-    /// passing the table itself along with `args` as function arguments.
-    ///
-    /// This function is deprecated since 0.3.1 in favor of [`call_method`]
-    /// in the `TableExt` trait.
-    ///
-    /// This might invoke the `__index` metamethod.
-    ///
-    /// [`call_method`]: trait.TableExt.html#tymethod.call_method
-    #[deprecated(since = "0.3.1", note = "Please use `call_method` instead")]
-    pub fn call<K, A, R>(&self, key: K, args: A) -> Result<R>
-    where
-        K: ToLua<'lua>,
-        A: ToLuaMulti<'lua>,
-        R: FromLuaMulti<'lua>,
-    {
-        let lua = self.0.lua;
-        let mut args = args.to_lua_multi(lua)?;
-        args.push_front(Value::Table(self.clone()));
-        self.get::<_, Function>(key)?.call(args)
-    }
-
     /// Compares two tables for equality.
     ///
     /// Tables are compared by reference first.
@@ -517,6 +496,8 @@ pub trait TableExt<'lua> {
     /// Gets the function associated to `key` from the table and asynchronously executes it,
     /// passing the table itself along with `args` as function arguments and returning Future.
     ///
+    /// Requires `feature = "async"`
+    ///
     /// This might invoke the `__index` metamethod.
     #[cfg(feature = "async")]
     fn call_async_method<'fut, K, A, R>(&self, key: K, args: A) -> LocalBoxFuture<'fut, Result<R>>
@@ -529,6 +510,8 @@ pub trait TableExt<'lua> {
     /// Gets the function associated to `key` from the table and asynchronously executes it,
     /// passing `args` as function arguments and returning Future.
     ///
+    /// Requires `feature = "async"`
+    ///
     /// This might invoke the `__index` metamethod.
     #[cfg(feature = "async")]
     fn call_async_function<'fut, K, A, R>(
@@ -627,9 +610,10 @@ where
                     lua.push_ref(&self.table);
                     lua.push_value(next_key)?;
 
-                    if protect_lua_closure(lua.state, 2, ffi::LUA_MULTRET, |state| {
+                    let next = protect_lua_closure(lua.state, 2, ffi::LUA_MULTRET, |state| {
                         ffi::lua_next(state, -2) != 0
-                    })? {
+                    })?;
+                    if next {
                         ffi::lua_pushvalue(lua.state, -2);
                         let key = lua.pop_value();
                         let value = lua.pop_value();
