@@ -1,6 +1,6 @@
 use std::cell::RefCell;
 use std::os::raw::{c_int, c_void};
-use std::rc::Rc;
+use std::sync::{Arc, Mutex};
 use std::{fmt, mem, ptr};
 
 #[cfg(feature = "async")]
@@ -8,6 +8,7 @@ use futures_core::future::LocalBoxFuture;
 
 use crate::error::Result;
 use crate::ffi;
+use crate::hook::Debug;
 use crate::lua::Lua;
 use crate::util::{assert_stack, StackGuard};
 use crate::value::MultiValue;
@@ -28,6 +29,18 @@ pub(crate) type Callback<'lua, 'a> =
 pub(crate) type AsyncCallback<'lua, 'a> =
     Box<dyn Fn(&'lua Lua, MultiValue<'lua>) -> LocalBoxFuture<'lua, Result<MultiValue<'lua>>> + 'a>;
 
+pub(crate) type HookCallback = Arc<RefCell<dyn FnMut(&Lua, Debug) -> Result<()>>>;
+
+#[cfg(feature = "send")]
+pub trait MaybeSend: Send {}
+#[cfg(feature = "send")]
+impl<T: Send> MaybeSend for T {}
+
+#[cfg(not(feature = "send"))]
+pub trait MaybeSend {}
+#[cfg(not(feature = "send"))]
+impl<T> MaybeSend for T {}
+
 /// An auto generated key into the Lua registry.
 ///
 /// This is a handle to a value stored inside the Lua registry.  It is not automatically
@@ -45,7 +58,7 @@ pub(crate) type AsyncCallback<'lua, 'a> =
 /// [`UserData::get_user_value`]: struct.UserData.html#method.get_user_value
 pub struct RegistryKey {
     pub(crate) registry_id: c_int,
-    pub(crate) unref_list: Rc<RefCell<Option<Vec<c_int>>>>,
+    pub(crate) unref_list: Arc<Mutex<Option<Vec<c_int>>>>,
 }
 
 impl fmt::Debug for RegistryKey {
@@ -56,7 +69,7 @@ impl fmt::Debug for RegistryKey {
 
 impl Drop for RegistryKey {
     fn drop(&mut self) {
-        let mut unref_list = mlua_expect!(self.unref_list.try_borrow_mut(), "unref list borrowed");
+        let mut unref_list = mlua_expect!(self.unref_list.lock(), "unref list poisoned");
         if let Some(list) = unref_list.as_mut() {
             list.push(self.registry_id);
         }
