@@ -1,7 +1,22 @@
 #![cfg(feature = "async")]
+#![cfg_attr(
+    all(feature = "luajit", target_os = "macos", target_arch = "x86_64"),
+    feature(link_args)
+)]
+
+#[cfg_attr(
+    all(feature = "luajit", target_os = "macos", target_arch = "x86_64"),
+    link_args = "-pagezero_size 10000 -image_base 100000000",
+    allow(unused_attributes)
+)]
+extern "system" {}
 
 use std::cell::Cell;
 use std::rc::Rc;
+use std::sync::{
+    atomic::{AtomicI64, Ordering},
+    Arc,
+};
 use std::time::Duration;
 
 use futures_timer::Delay;
@@ -180,7 +195,7 @@ async fn test_async_thread_stream() -> Result<()> {
 async fn test_async_thread() -> Result<()> {
     let lua = Lua::new();
 
-    let cnt = Rc::new(10); // sleep 10ms
+    let cnt = Arc::new(10); // sleep 10ms
     let cnt2 = cnt.clone();
     let f = lua.create_async_function(move |_lua, ()| {
         let cnt3 = cnt2.clone();
@@ -194,9 +209,9 @@ async fn test_async_thread() -> Result<()> {
 
     assert_eq!(res, "done");
 
-    assert_eq!(Rc::strong_count(&cnt), 2);
+    assert_eq!(Arc::strong_count(&cnt), 2);
     lua.gc_collect()?; // thread_s is non-resumable and subject to garbage collection
-    assert_eq!(Rc::strong_count(&cnt), 1);
+    assert_eq!(Arc::strong_count(&cnt), 1);
 
     Ok(())
 }
@@ -252,18 +267,18 @@ async fn test_async_table() -> Result<()> {
 #[tokio::test]
 async fn test_async_userdata() -> Result<()> {
     #[derive(Clone)]
-    struct MyUserData(Rc<Cell<i64>>);
+    struct MyUserData(Arc<AtomicI64>);
 
     impl UserData for MyUserData {
         fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
             methods.add_async_method("get_value", |_, data, ()| async move {
                 Delay::new(Duration::from_millis(10)).await;
-                Ok(data.0.get())
+                Ok(data.0.load(Ordering::Relaxed))
             });
 
             methods.add_async_method("set_value", |_, data, n| async move {
                 Delay::new(Duration::from_millis(10)).await;
-                data.0.set(n);
+                data.0.store(n, Ordering::Relaxed);
                 Ok(())
             });
 
@@ -277,15 +292,15 @@ async fn test_async_userdata() -> Result<()> {
     let lua = Lua::new();
     let globals = lua.globals();
 
-    let userdata = lua.create_userdata(MyUserData(Rc::new(Cell::new(11))))?;
+    let userdata = lua.create_userdata(MyUserData(Arc::new(AtomicI64::new(11))))?;
     globals.set("userdata", userdata.clone())?;
 
     lua.load(
         r#"
         assert(userdata:get_value() == 11)
         userdata:set_value(12)
-        assert(userdata:get_value() == 12)
         assert(userdata.sleep(5) == "elapsed:5ms")
+        assert(userdata:get_value() == 12)
     "#,
     )
     .exec_async()
@@ -293,3 +308,106 @@ async fn test_async_userdata() -> Result<()> {
 
     Ok(())
 }
+
+#[tokio::test]
+async fn test_async_scope() -> Result<()> {
+    let ref lua = Lua::new();
+
+    let ref rc = Rc::new(Cell::new(0));
+
+    let fut = lua.async_scope(|scope| async move {
+        let f = scope.create_async_function(move |_, n: u64| {
+            let rc2 = rc.clone();
+            async move {
+                rc2.set(42);
+                Delay::new(Duration::from_millis(n)).await;
+                assert_eq!(Rc::strong_count(&rc2), 2);
+                Ok(())
+            }
+        })?;
+
+        lua.globals().set("f", f.clone())?;
+
+        assert_eq!(Rc::strong_count(rc), 1);
+        let _ = f.call_async::<u64, ()>(10).await?;
+        assert_eq!(Rc::strong_count(rc), 1);
+
+        Ok(())
+    });
+
+    assert_eq!(Rc::strong_count(rc), 1);
+    let _ = fut.await?;
+
+    match lua
+        .globals()
+        .get::<_, Function>("f")?
+        .call_async::<_, ()>(10)
+        .await
+    {
+        Err(Error::CallbackError { ref cause, .. }) => match cause.as_ref() {
+            Error::CallbackDestructed => {}
+            e => panic!("expected `CallbackDestructed` error cause, got {:?}", e),
+        },
+        r => panic!("improper return for destructed function: {:?}", r),
+    };
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn test_async_scope_userdata() -> Result<()> {
+    #[derive(Clone)]
+    struct MyUserData(Arc<AtomicI64>);
+
+    impl UserData for MyUserData {
+        fn add_methods<'lua, M: UserDataMethods<'lua, Self>>(methods: &mut M) {
+            methods.add_async_method("get_value", |_, data, ()| async move {
+                Delay::new(Duration::from_millis(10)).await;
+                Ok(data.0.load(Ordering::Relaxed))
+            });
+
+            methods.add_async_method("set_value", |_, data, n| async move {
+                Delay::new(Duration::from_millis(10)).await;
+                data.0.store(n, Ordering::Relaxed);
+                Ok(())
+            });
+
+            methods.add_async_function("sleep", |_, n| async move {
+                Delay::new(Duration::from_millis(n)).await;
+                Ok(format!("elapsed:{}ms", n))
+            });
+        }
+    }
+
+    let ref lua = Lua::new();
+
+    let ref arc = Arc::new(AtomicI64::new(11));
+
+    lua.async_scope(|scope| async move {
+        let ud = scope.create_userdata(MyUserData(arc.clone()))?;
+        lua.globals().set("userdata", ud)?;
+        lua.load(
+            r#"
+            assert(userdata:get_value() == 11)
+            userdata:set_value(12)
+            assert(userdata.sleep(5) == "elapsed:5ms")
+            assert(userdata:get_value() == 12)
+        "#,
+        )
+        .exec_async()
+        .await
+    })
+    .await?;
+
+    assert_eq!(Arc::strong_count(arc), 1);
+
+    match lua.load("userdata:get_value()").exec_async().await {
+        Err(Error::CallbackError { ref cause, .. }) => match cause.as_ref() {
+            Error::CallbackDestructed => {}
+            e => panic!("expected `CallbackDestructed` error cause, got {:?}", e),
+        },
+        r => panic!("improper return for destructed userdata: {:?}", r),
+    };
+
+    Ok(())
+}
