@@ -1,3 +1,15 @@
+#![cfg_attr(
+    all(feature = "luajit", target_os = "macos", target_arch = "x86_64"),
+    feature(link_args)
+)]
+
+#[cfg_attr(
+    all(feature = "luajit", target_os = "macos", target_arch = "x86_64"),
+    link_args = "-pagezero_size 10000 -image_base 100000000",
+    allow(unused_attributes)
+)]
+extern "system" {}
+
 use std::cell::Cell;
 use std::rc::Rc;
 
@@ -44,10 +56,8 @@ fn scope_drop() -> Result<()> {
     let rc = Rc::new(());
 
     lua.scope(|scope| {
-        lua.globals().set(
-            "test",
-            scope.create_static_userdata(MyUserdata(rc.clone()))?,
-        )?;
+        lua.globals()
+            .set("test", scope.create_userdata(MyUserdata(rc.clone()))?)?;
         assert_eq!(Rc::strong_count(&rc), 2);
         Ok(())
     })?;
@@ -198,7 +208,6 @@ fn scope_userdata_mismatch() -> Result<()> {
             a.inc(a)
             b.inc(b)
         end
-
         function bad(a, b)
             a.inc(b)
         end
